#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys

sys.path.append("../../lib/")

from leda_python import leda
from leda_python.mbus import MbusBase
import hashlib
import logging
import threading
import json
import time
import os
import unittest

_logger = logging.getLogger(__name__)

driverObj = leda.LedaModule()


def config_stop():
    os.system(
        "ps -ef | grep \"ut_config\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>>/dev/null")


def config_start():
    config_stop()
    os.system("./startConfig.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def configTimeOut_start():
    config_stop()
    os.system("./startConfig_timeOut.sh &  1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def configErrCode_start():
    config_stop()
    os.system("./startConfig_errCode.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimu_stop():
    os.system(
        "ps -ef | grep \"ut_dimu\" | grep -v \"grep\" | awk \'{print $2}\' | xargs kill -2 1>> /dev/null 2>>/dev/null")


def dimu_start():
    dimu_stop()
    os.system("./startDimu.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimuTimeOut_start():
    dimu_stop()
    os.system("./startDimu_timeOut.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimuErrCode_start():
    dimu_stop()
    os.system("./startDimu_errCode.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


def dimuRetParamsInvalid_start():
    dimu_stop()
    os.system("./startDimu_retParamsInvalid.sh & 1>> /dev/null 2>>/dev/null")
    time.sleep(3)


class APIUnittest(unittest.TestCase):

    def test_getConfig_beforeInit(self):

        dimu_start()

        config_start()
        try:
            driverObj.getPdInfo()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_registerDriver_beforeInit(self):
        dimu_start()
        try:
            driverObj._registerModule()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    # @unittest.skip('i dont test the case')
    def test_unregisterDriver_beforeInit(self):
        dimu_start()
        try:

            driverObj._unregisterModule()
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    def test_SubConfig_beforeInit(self):

        dimu_start()

        class ConfigCallback(leda.LedaConfigCallback):
            def deviceConfigCB(self, moduleName, moduleConfig):
                print("configCallback: moduleName %s, moduleConfigMsg...ok!!: %s", moduleName, moduleConfig)

        config_start()

        try:
            driverObj.registerDeviceConfigCallback(ConfigCallback())
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    def test_registerDevice_beforeInit(self):
        dimu_start()
        driver_name = "abc123"

        device_name = 'device123'
        product_key = 'abcdefghigk'

        profile_dict = {
            "properties": [],
            "services": [],
            "events": []
        }

        profile = json.dumps(profile_dict)

        class Demo_device(leda.BaseDeviceCallback):
            def __init__(self):
                self.name = "deviceMbus"

            def callService(self, name, input):
                pass

            def getprofile_cb(self):
                pass

            def getProperties(self, input):
                pass

            def setProperties(self, input):
                pass

        bus_callback_object = Demo_device()
        try:
            driverObj.deviceRegister(device_name, product_key, profile, bus_callback_object)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)

    def test_watchdog_beforeInit(self):
        dimu_start()
        try:
            driverObj.feedDog("thread_name", 123)
            assertID = False
        except:
            assertID = True

        self.assertEqual(assertID, True)


if (__name__ == "__main__"):
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s-%(threadName)-9s-%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s')

    _logger.debug("demo driver testing...")

    # suite = unittest.TestSuite()#创建测试套件
    unittest.main(verbosity=2)
