# -*- coding: utf-8 -*-
import time, os, json
import serial
import struct
import lethingaccesssdk

serialPort = '/dev/ttyM0'  # 串口
baudRate = 115200  # 波特率
serial = serial.Serial(port=serialPort, baudrate=baudRate)
data_bytes = bytearray()
data_snapshots = {}
device_list_g = {}
high_temperature_threshold = 60.00


class GrmDevice(lethingaccesssdk.ThingCallback):
    productKey = ''
    deviceName = ''
    localID = ''

    def __init__(self, pk, dn, local_id):
        self.productKey = pk
        self.deviceName = dn
        self.localID = local_id
        print('-- Create new GRM device: ProductKey=%s DeviceName=%s .' % (self.productKey, self.deviceName))

    def callService(self, name, input_value):
        return 0, {}

    def getProperties(self, input_value):
        print('-- getProperties input_value: ', input_value)
        if input_value[0] == "temperature":
            try:
                device_id = str(self.localID)
                return 0, {input_value[0]: data_snapshots[device_id]['real_value']}
            except Exception as e:
                print('-- [Warn] getProperties failed.', e)
                return 0, {input_value[0]: 0}
        else:
            return -1

    def setProperties(self, input_value):
        return 0, {}


def open_serial():
    if not serial.isOpen():
        serial.open()
    if serial.isOpen():
        print(serial)
        print("-- {} open success".format(serialPort))
        return True
    else:
        print("-- {} open failed".format(serialPort))
        return False


def hexshow(data):
    # frameData = ' '.join(['0x'+ format(x, '02x') for x in data])
    frameData = ' '.join([format(x, '02x') for x in data])
    localtime = time.asctime(time.localtime(time.time()))
    print("Time:", localtime)
    print('Data type:', type(data))
    print('Receive:', frameData)
    return


###################################################
# 功 能: 将需要发送的字符串以hex形式发送
# 参 数: 待发送的数据
# 返 回: 转换后的数据
###################################################
def hexsend(string_data=''):
    hex_data = string_data.decode("hex")
    return hex_data


def grm_parse():
    global data_bytes
    data_len = len(data_bytes)
    print("-- IN DataLen:", data_len)
    if data_len <= 5:
        print("-- DataLen is too short: {}. Continue.".format(data_len))
        time.sleep(0.1)
        return False

    i = 0
    while i < (data_len - 1):
        if data_bytes[i] == 0xFE:
            # frame_len = int(data_bytes[i+1], 16)
            frame_len = data_bytes[i + 1]
            print('frame_len:', frame_len)
            if frame_len > 82:
                print('-- [Error] Frame length illegal: {}, should < 82 bytes.'.format(frame_len))
                i = i + 2
                continue
            if data_len < (frame_len + 5):
                print("-- [Error] DataLen is too short: {}, frame_len: {}. Continue.".format(data_len, frame_len))
                time.sleep(0.1)
                return False

            cmd_area = struct.unpack('!H', data_bytes[i + 2:i + 4])[0]
            send_addr = struct.unpack('!H', data_bytes[i + 4:i + 6])[0]
            protocol_type = struct.unpack('!H', data_bytes[i + 6:i + 8])[0]
            if protocol_type != 1:
                print("-- [Error] Protocol type: {}, should be 1. Ignore this message!".format(protocol_type))
                i = i + 4 + frame_len + 1
                continue
            vendor_code = struct.unpack('!H', data_bytes[i + 8:i + 10])[0]
            instrument_type = struct.unpack('!H', data_bytes[i + 10:i + 12])[0]
            location = data_bytes[i + 12]
            instrument_order = data_bytes[i + 13]
            function_code = struct.unpack('!H', data_bytes[i + 14:i + 16])[0]
            communication_efficiency = data_bytes[i + 16]
            battery_left = data_bytes[i + 17]
            sleep_time = struct.unpack('!H', data_bytes[i + 18:i + 20])[0]
            instrument_status = struct.unpack('!H', data_bytes[i + 20:i + 22])[0]
            running_time = struct.unpack('!H', data_bytes[i + 22:i + 24])[0]
            device_id = str(instrument_order)
            data_snapshots[device_id] = {}
            if instrument_type == 3 or instrument_type == 2:
                real_value = struct.unpack('!H', data_bytes[i + 24:i + 26])[0]
                data_snapshots[device_id]['real_value'] = real_value / 100
            else:
                print("-- [Error] Instrument type: {} is not supportted yet. Ignore this message!".format(
                    instrument_type))
                i = i + 4 + frame_len + 1
                continue

            i = i + 4 + frame_len + 1

            # Report property
            if instrument_type == 3:
                propertiesDict = {"temperature": real_value / 100}
                device_list_g[device_id]['client'].reportProperties(propertiesDict)
                # Report event
                if (real_value / 100) >= high_temperature_threshold:
                    device_list_g[device_id]['client'].reportEvent('high_temperature',
                                                                   {'temperature': real_value / 100})
            elif instrument_type == 2:
                propertiesDict = {"pressure": real_value / 100}
                device_list_g[device_id]['client'].reportProperties(propertiesDict)

            # Save last value to snapshot
            data_snapshots[device_id]['frame_len'] = frame_len
            data_snapshots[device_id]['command_area'] = cmd_area
            data_snapshots[device_id]['send_addr'] = send_addr
            data_snapshots[device_id]['protocol_type'] = protocol_type
            data_snapshots[device_id]['vendor_code'] = vendor_code
            data_snapshots[device_id]['instrument_type'] = instrument_type
            data_snapshots[device_id]['location'] = location
            data_snapshots[device_id]['instrument_order'] = instrument_order
            data_snapshots[device_id]['function_code'] = function_code
            data_snapshots[device_id]['location'] = location
            data_snapshots[device_id]['communication_efficiency'] = communication_efficiency
            data_snapshots[device_id]['battery_left'] = battery_left
            data_snapshots[device_id]['sleep_time'] = sleep_time
            data_snapshots[device_id]['instrument_status'] = instrument_status
            data_snapshots[device_id]['running_time'] = running_time
            print("data_snapshots: ", data_snapshots)
        else:
            i = i + 1
    data_bytes[0:i] = b''
    print("-- OUT DataLen:", len(data_bytes))
    return True


def serial_polling():
    global data_bytes
    print('-- Start polling the serial port.')
    try:
        while True:
            count = serial.inWaiting()
            if count > 0:
                print("-- Serial wait return: ", count)
                rcv_str = serial.read(count)
                if rcv_str != b'':
                    data_bytes = data_bytes + rcv_str
                    hexshow(data_bytes)
                    grm_parse()
            else:
                time.sleep(1)
    except Exception as e:
        print("-- [Error] Serial polling failed.", e)
        if serial != None:
            serial.close()


def register_devices():
    infos = lethingaccesssdk.Config().getThingInfos()
    for info in infos:
        try:
            pk = info['productKey']
            dn = info['deviceName']
            custom = info['custom'];
            print('custom: ', custom, type(custom))
            dict_custom = json.loads(custom)
            # parse custom data
            local_id = dict_custom['id']  # mapping local_id to PK_DN
            local_id_str = str(local_id)
            device_callback = GrmDevice(pk, dn, local_id_str)
            client = lethingaccesssdk.ThingAccessClient(info)
            client.registerAndonline(device_callback)
            device_list_g[local_id_str] = {}
            device_list_g[local_id_str]['productKey'] = pk
            device_list_g[local_id_str]['deviceName'] = dn
            device_list_g[local_id_str]['client'] = client
            device_list_g[local_id_str]['callback'] = device_callback
        except Exception as e:
            print("-- [Error] Register devices failed.", e)


register_devices()
open_serial()
serial_polling()


def handler(event, context):
    return 'OK'
